#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define SUSCEPTIBLE 0
#define INFECTED 1
#define RECOVERING 2
#define DEAD 3

struct node
{
        int data;
        struct node *next;
} **neigh_list;

/* The following parameters are all external */
double INF_PROB,DEATH_PROB,RECOVERY_RATE;
int *neighbors;

int histo_duration[10000],histo_mass[10000];

int sirn();
void add_node(),remove_node_int();

/* The following are Numerical Recipes routines for vector allocation-deallocation */

void nrerror(char error_text[])
/* Numerical Recipes standard error handler */
{
	fprintf(stderr,"Run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}

int *ivector(long nl, long nh)
/* allocate an int vector with subscript range v[nl..nh] */
{
	int *v;
	v=(int *)malloc((size_t) ((nh-nl+1+1)*sizeof(int)));
	if (!v) nrerror("allocation failure in ivector()");
	return v-nl+1;
}

void free_ivector(int *v, long nl, long nh)
/* free an int vector allocated with ivector() */
{
	free((char *) (v+nl-1));
}


int main(argc,argv)
int argc;
char *argv[];
{
	int reps,i,j,SEED,runs,cluster_size,mass,origin,den;
	char name[100],name1[100];
	double sum;
	int read_the_network();
	FILE *fp;
	void clear_lists();
	
	if(argc==6)
	{
		SEED=atoi(argv[2]);
		INF_PROB=atof(argv[3]);
		DEATH_PROB=atof(argv[4]);
		RECOVERY_RATE=atof(argv[5]);
	}
	else
        {
                printf("Usage: %s filename seed infection_probability death_probability recovery_rate\n",argv[0]);
		return(-1);
	}

	for(i=0;i<10000;i++) {histo_duration[i]=0;histo_mass[i]=0;}

	den=0;
	/* Read the network. cluster_size is the real network size */
	cluster_size=read_the_network(argv[1]);

	srand48(SEED);

	/* Choose the origin. Here all nodes are used as origins. Can change to select random sub-set */
	for(origin=0;origin<cluster_size;origin++)
	{
		/* Number of repetitions per origin */
		for(reps=0;reps<5;reps++)
		{
			mass=sirn(cluster_size,origin);
			den++;
		}
	}
	clear_lists(cluster_size);

	sprintf(name,"SIRDS_r%.2f_b%.2f_f%.2f.dat",RECOVERY_RATE,INF_PROB,DEATH_PROB);
	fp=fopen(name,"w");
	for(i=0;i<=cluster_size;i++) fprintf(fp,"%g\t%g\t%g\n",i/(double)cluster_size,histo_mass[i]/(double)den,histo_duration[i]/(double)den);
	fclose(fp);

	return(0);
}

int sirn(size,origin)
int origin,size;
{
        int i,j,n_inf,n,c2,total,mass,dead,n_recov;
        int *just_inf,*infected,*status;
        struct node *temp,*t2,*recovering;
	int t,maxindex;

        infected=ivector(0,size-1);
        just_inf=ivector(0,size-1);
	status=ivector(0,size-1);

	recovering = (struct node *)malloc(sizeof(struct node));
	recovering = NULL;

	for(i=0;i<size;i++) {status[i]=SUSCEPTIBLE;}

        n_inf=1;
        infected[0]=origin;
	status[origin]=INFECTED;
	mass=1;
	dead=0;
	n_recov=0;
	t=0;
		
        do
        {
		t++;
		temp=recovering;
		while(temp!=NULL)
		{
			t2=temp->next;
			if(drand48()<RECOVERY_RATE)
			{
				status[temp->data]=SUSCEPTIBLE;
				remove_node_int(temp->data,&recovering);
				n_recov--;
			}
			temp=t2;
		}
                total=0;
                for(i=0;i<n_inf;i++)
                {
                        n=infected[i];
                        temp=neigh_list[n];
                        while(temp!=NULL)
                        {
				c2=temp->data;
				if(status[c2]==SUSCEPTIBLE)
				{
					if(drand48()<INF_PROB)
	                                {
						status[c2]=INFECTED;
        	                                just_inf[total]=c2;
	                                        total++;
        	                        }
				}
				temp=temp->next;
                        }
			if(drand48()<DEATH_PROB)
			{
				status[n]=DEAD;
				dead++;
			}
			else
			{
				status[n]=RECOVERING;
				add_node(n,&recovering);
				n_recov++;
			}
                }
                n_inf=total;
		mass+=total;
                for(i=0;i<total;i++) infected[i]=just_inf[i];
        } while(total!=0);

	histo_mass[dead]++;
	histo_duration[t]++;
        temp=recovering;
        while(temp!=NULL)
        {
                t2=temp->next;
                free(temp);
                temp=t2;
        }

	free_ivector(status,0,size-1);
        free_ivector(just_inf,0,size-1);
        free_ivector(infected,0,size-1);

        return(dead);
}


int read_the_network(name)
char name[100];
{
        int i,n,k,size;
        FILE *fp;

        fp=fopen(name,"r");
        fscanf(fp,"%d\n",&size);
        neighbors=ivector(0,size-1);
        for(i=0;i<size;i++) neighbors[i]=0;
        /* Allocate neighbor lists memory for each node */
        neigh_list=(struct node **)malloc(size*sizeof(struct node));
        for(i=0;i<size;i++)
        {
                neigh_list[i]=NULL;
        }

        while(!feof(fp))
        {
                fscanf(fp,"%d\t%d\t",&i,&n);
                neighbors[i]=n;
                for(k=0;k<neighbors[i];k++)
                {
                        fscanf(fp,"%d\t",&n);
                        add_node(n,&neigh_list[i]);
                }
                fscanf(fp,"\n");
        }
        fclose(fp);

        return(size);
}

void clear_lists(cluster_size)
int cluster_size;
{
	struct node *current,*cur2;
	int i;
	
	for(i=0;i<cluster_size;i++)
	{
		current=neigh_list[i];
               	while(current!=NULL)
               	{
                       	cur2=current->next;
                       	free(current);
                       	current=cur2;
	       	}
       	       	neigh_list[i]=NULL;
	}
	free(neigh_list);
	free_ivector(neighbors,0,cluster_size-1);
	return;
}

void add_node(i,headRef)
int i;
struct node** headRef;
{ 
        struct node *newnode;
 
        newnode = (struct node *)malloc(sizeof(struct node));
        newnode->data=i;
        newnode->next=*headRef;
 
 	*headRef=newnode;
	
        return;
}

void remove_node_int(tobedeleted,headRefFrom)
int tobedeleted;
struct node** headRefFrom;
{
        struct node *p,*q;

        p=*headRefFrom;
        if(p->data==tobedeleted) /* If it is the first node in the list ... */
        {
                *headRefFrom=p->next;
		q=p;
        }
        else /* It is not the first node in the From list ... */
        {
                while(p->next->data!=tobedeleted) p=p->next; /* locate the node */
 
 		q=p->next;
                p->next=q->next;
        }
        free(q);
        return;
}
